using Microsoft.AspNetCore.Mvc;
using System;
using System.Threading.Tasks;
using WebApplication1.Models;
using WebApplication1.Services;
using WebApplication1.ViewModels;

namespace WebApplication1.Controllers
{
    public class PurchaseInformationController : Controller
    {
        private readonly PurchaseInformationService _service;

        public PurchaseInformationController(PurchaseInformationService service)
        {
            _service = service;
        }

        // 1.初期表示
        public async Task<IActionResult> Index()
        {
            // (comment Start)1-1.本画面の使用権限があるか確認する（共通部品を参照）(comment End)
            if (!await _service.CheckUserPermissionAsync())
            {
                return Unauthorized(); // 権限がない場合は未承認(401)を返す
            }

            var viewModel = new PurchaseInformationViewModel
            {
                Suppliers = await _service.GetAllSuppliersAsync(),
                Persons = await _service.GetAllPersonsAsync(),
                ItemCategories = await _service.GetAllItemCategoriesAsync(),
                StartDate = DateTime.Now.AddDays(-DateTime.Now.Day + 1), // 当月月初
                EndDate = DateTime.Now.AddMonths(1).AddDays(-DateTime.Now.Day) // 当月月末
            };

            return View(viewModel);
        }

        // 2.検索処理
        [HttpPost]
        public async Task<IActionResult> Search(
            string supplierCode, string personCode, DateTime? startDate, DateTime? endDate, 
            string itemCategory, string itemName)
        {
            var (purchaseInformationList, errorMessage) = await _service.SearchPurchaseInformationAsync(
                supplierCode, personCode, startDate, endDate, itemCategory, itemName);

            if (!string.IsNullOrEmpty(errorMessage))
            {
                return Json(new { ErrorMessage = errorMessage });
            }

            var subtotalBySupplier = await _service.CalculateSubtotalBySupplierAsync(purchaseInformationList);
            var totalAmount = await _service.CalculateTotalAmountAsync(purchaseInformationList);

            var result = new
            {
                PurchaseInformationList = purchaseInformationList,
                SubtotalBySupplier = subtotalBySupplier,
                TotalAmount = totalAmount
            };

            return Json(result);
        }

        // 3.仕入先リンク押下
        public IActionResult SupplierLinkClicked(string supplierCode)
        {
            // 3-1.画面項目名の仕入先コードを引数として、仕入先照会画面をポップアップ表示する。
            // 仕入先照会画面のポップアップ表示処理を想定
            return View("SupplierInquiryPopup", supplierCode); // 仮実装
        }
    }
}