package com.example.controller;

import com.example.entity.PurchaseInformationEntity;
import com.example.form.SearchForm;
import com.example.service.PurchaseInformationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import java.util.List;

/**
 * 仕入情報照会画面のコントローラー
 */
@Controller
public class PurchaseInformationController {

    @Autowired
    private PurchaseInformationService purchaseInformationService;

    /**
     * 初期表示処理
     * 
     * @param model モデル
     * @return 仕入情報照会画面のパス
     */
    @GetMapping("/search")
    public String init(Model model) {
        // (comment Start)1-1.本画面の使用権限があるか確認する（共通部品を参照）(comment End)
        // 1-2.仕入先のリスト情報を仕入先マスタから全件取得し、仕入先の選択肢として設定する。
        // 1-3.仕入担当者のリスト情報を担当者マスタから全件取得し、仕入先担当者の選択肢として設定する。
        // 1-4.仕入開始日に当月の月初日付を表示する。
        // 1-5.仕入終了日に当月の月末日付を表示する。
        // 1-6.商品種別のリスト情報を商品種別マスタから全件取得し、商品種別の選択肢として設定する。
        model.addAttribute("searchForm", new SearchForm());
        return "search";
    }

    /**
     * 検索処理
     * 
     * @param searchForm 検索フォーム
     * @param result バインディング結果
     * @param model モデル
     * @return 仕入情報照会画面のパス
     */
    @PostMapping("/search")
    public String search(@Validated SearchForm searchForm, BindingResult result, Model model) {
        if (result.hasErrors()) {
            // フォームのバリデーションエラーがある場合
            return "search";
        }

        // 2-1.仕入開始日、仕入終了日のどちらも入力されていない場合は「検索条件を指定してください。」とエラーメッセージを返却する。
        if (searchForm.getPurchaseStartDate() == null && searchForm.getPurchaseEndDate() == null) {
            model.addAttribute("errorMessage", "検索条件を指定してください。");
            return "search";
        }

        // 2-2.仕入開始日＞仕入終了日の場合は「仕入日の検索条件の指定に誤りがあります。」とエラーメッセージを返却する。
        if (searchForm.getPurchaseStartDate() != null && searchForm.getPurchaseEndDate() != null &&
            searchForm.getPurchaseStartDate().after(searchForm.getPurchaseEndDate())) {
            model.addAttribute("errorMessage", "仕入日の検索条件の指定に誤りがあります。");
            return "search";
        }

        // 2-3.検索条件に基づいて仕入情報を取得する。
        List<PurchaseInformationEntity> purchaseInformationList = purchaseInformationService.searchPurchaseInformation(
                searchForm.getSupplier(),
                searchForm.getPurchasePerson(),
                searchForm.getPurchaseStartDate(),
                searchForm.getPurchaseEndDate(),
                searchForm.getItemCategory(),
                searchForm.getItemName());

        // 2-6.検索結果が0件の場合は、「検索結果が存在しませんでした。」とエラーメッセージを返却する。
        if (purchaseInformationList.isEmpty()) {
            model.addAttribute("errorMessage", "検索結果が存在しませんでした。");
            return "search";
        }

        // 2-4.画面項目名の小計金額に前記で取得した仕入情報をループし、仕入先毎に(仕入単価×仕入数量)を集計し、仕入先毎に集計した結果値を表示する。
        // 2-5.画面項目名の合計金額に前記で取得した仕入情報をループし、全ての(仕入単価×仕入数量)を集計し、集計した結果値を表示する。
        // ここで小計金額と合計金額の計算処理を行う（実装は省略）

        model.addAttribute("purchaseInformationList", purchaseInformationList);
        return "search";
    }
}