from django.views import View
from django.shortcuts import render
from .models import Supplier, Person, ItemCategory, PurchaseInformation
from django.db.models import Sum
from django.core.exceptions import ValidationError
from django.utils import timezone
import calendar

class PurchaseInquiryView(View):
    template_name = 'purchase_inquiry.html'

    def get(self, request, *args, **kwargs):
        # (comment Start)1-1.本画面の使用権限があるか確認する（共通部品を参照）(comment End)
        suppliers = Supplier.objects.all()
        persons = Person.objects.all()
        item_categories = ItemCategory.objects.all()
        today = timezone.now().date()
        first_day_of_month = today.replace(day=1)
        last_day_of_month = today.replace(day=calendar.monthrange(today.year, today.month)[1])
        context = {
            'suppliers': suppliers,
            'persons': persons,
            'item_categories': item_categories,
            'start_date': first_day_of_month,
            'end_date': last_day_of_month,
        }
        return render(request, self.template_name, context)

    def post(self, request, *args, **kwargs):
        # 検索条件を取得
        supplier_code = request.POST.get('supplier')
        person_code = request.POST.get('person')
        start_date = request.POST.get('start_date')
        end_date = request.POST.get('end_date')
        item_category_code = request.POST.get('item_category')
        item_name = request.POST.get('item_name')

        # (comment Start)2-1.仕入開始日、仕入終了日のどちらも入力されていない場合は「検索条件を指定してください。」とエラーメッセージを返却する。(comment End)
        if not start_date and not end_date:
            return render(request, self.template_name, {'error_message': '検索条件を指定してください。'})

        # (comment Start)2-2.仕入開始日＞仕入終了日の場合は「仕入日の検索条件の指定に誤りがあります。」とエラーメッセージを返却する。(comment End)
        if start_date and end_date and start_date > end_date:
            return render(request, self.template_name, {'error_message': '仕入日の検索条件の指定に誤りがあります。'})

        # 検索条件に基づいて仕入情報を取得
        purchase_info_query = PurchaseInformation.objects.all()
        if supplier_code:
            purchase_info_query = purchase_info_query.filter(supplier_code=supplier_code)
        if person_code:
            purchase_info_query = purchase_info_query.filter(purchase_pic=person_code)
        if start_date:
            purchase_info_query = purchase_info_query.filter(purchase_date__gte=start_date)
        if end_date:
            purchase_info_query = purchase_info_query.filter(purchase_date__lte=end_date)
        if item_category_code:
            purchase_info_query = purchase_info_query.filter(item_code__item_category=item_category_code)
        if item_name:
            purchase_info_query = purchase_info_query.filter(item_code__item_name__icontains=item_name)

        # (comment Start)2-3.仕入情報と仕入先マスタと担当者マスタと商品マスタ、商品種別マスタを結合し、ソート順は仕入先コード、仕入日として仕入情報を取得する。(comment End)
        purchase_info_query = purchase_info_query.select_related('supplier_code', 'purchase_pic', 'item_code__item_category').order_by('supplier_code', 'purchase_date')

        # (comment Start)2-4.画面項目名の小計金額に前記で取得した仕入情報をループし、仕入先毎に(仕入単価×仕入数量)を集計し、仕入先毎に集計した結果値を表示する。(comment End)
        subtotal_by_supplier = purchase_info_query.values('supplier_code').annotate(subtotal=Sum('purchase_unit_price' * 'purchase_quantity'))

        # (comment Start)2-5.画面項目名の合計金額に前記で取得した仕入情報をループし、全ての(仕入単価×仕入数量)を集計し、集計した結果値を表示する。(comment End)
        total_amount = purchase_info_query.aggregate(total=Sum('purchase_unit_price' * 'purchase_quantity'))['total']

        # (comment Start)2-6.検索結果が0件の場合は、「検索結果が存在しませんでした。」とエラーメッセージを返却する。(comment End)
        if not purchase_info_query.exists():
            return render(request, self.template_name, {'error_message': '検索結果が存在しませんでした。'})

        context = {
            'purchase_info': purchase_info_query,
            'subtotal_by_supplier': subtotal_by_supplier,
            'total_amount': total_amount,
        }
        return render(request, self.template_name, context)